import common
import random

class City:
    
    # The ROAD_* form means "There is a road to my *"
    # I.e. ROAD_NORTH | ROAD_SOUTH = road to north and
    # south.
    ROAD_NORTH = 1
    ROAD_SOUTH = 2
    ROAD_EAST = 4
    ROAD_WEST = 8
    ROAD_MASK = 0xF
    ROAD = 0
    ROADS = 16
    GRASS = 16
    SAFEHOUSE = 32
    
    def __init__(self,data=[]):
        self.data = data
    
    def at(self,x,y):
        x = int(x)
        y = int(y)
        if (x >= self.width or x < 0 or
             y >= self.height or y < 0):
            raise IndexError("x,y %f,%f out of range" % (x,y))
        return self.data[y * self.width + x]
    
    def atPut(self,x,y,datum):
        self.data[y * self.width + x] = datum
    
    def chooseSafehouse(self):
        x,y = (random.randint(0,self.width-1),
                random.randint(0,self.width-1))
        while (self.isRoad(self.at(x,y))):
            x,y = (random.randint(0,self.width-1),
                random.randint(0,self.width-1))
        self.atPut(x,y,self.SAFEHOUSE)
    
    def createBetterRoads(self):
        for y in range(self.height):
            for x in range(self.width):
                result = self.ROAD
                if(self.at(x,y) != self.GRASS):
                    if y > 0:
                        if self.at(x,y-1) != self.GRASS:
                            result |= self.ROAD_NORTH
                            #print "Setting ROAD_NORTH"
                    if y < self.height -1:
                        if self.at(x,y+1) != self.GRASS:
                            result |= self.ROAD_SOUTH
                            #print "Setting ROAD_SOUTH"
                    if x > 0:
                        if self.at(x-1,y) != self.GRASS:
                            result |= self.ROAD_WEST
                            #print "Setting ROAD_WEST"
                    if x < self.width-1:
                        if self.at(x+1,y) != self.GRASS:
                            #print "Setting ROAD_EAST"
                            result |= self.ROAD_EAST
                    #print "Setting to",result
                    self.atPut(x,y,result)    
                    
    def createCustomCity(self,width,height):
        self.createGrassCity(width,height)
        self.atPut(0,0,self.ROAD)
        self.createBetterRoads()
                    
    def createGrassCity(self,width,height):
        self.width = width
        self.height= height
        for y in range(height):
            for x in range(width):
                self.data.append(self.GRASS)
                
        #self.createBetterRoads()
        
    def createGridCity(self,width,height):
        self.width = width
        self.height= height
        for y in range(height):
            for x in range(width):
                if(x % 3 == 0 or y % 3 == 0):
                    self.data.append(self.ROAD)
                else:
                    self.data.append(self.GRASS)
                    
        self.createBetterRoads()
        
        self.chooseSafehouse()
    
    def exitsFromRoad(self,tileX,tileY):
        tileType = self.at(tileX,tileY)
        targets = []
        
        if (tileType & self.ROAD_NORTH):
            targets.append( [0, -1] )
        if (tileType & self.ROAD_SOUTH):
            targets.append( [0, 1] )
        if (tileType & self.ROAD_EAST):
            targets.append( [1, 0] )
        if (tileType & self.ROAD_WEST):
            targets.append( [-1, 0] )
        if len(targets) == 0:
            targets.append( [0, 0] )
        
        return targets
    
    def getFrictionFor(self,car):
        x,y = car.x,car.y
        tile = self.getAt(x,y)
        if tile == self.GRASS:
            return 0.3
        else:
            return 0.1
    
    def getRandomRoad(self):
        roads = []
        for index in range(len(self.data)):
            if self.data[index] & self.ROAD_MASK:
                roads.append(index)
        if roads:
            index = random.choice(roads)
            tileX = index % self.width
            tileY = index / self.height
            pixelX,pixelY = self.tileToWorldCentered(tileX,tileY)
        else:
            pixelX = 0
            pixelY = 0
        
        return pixelX,pixelY
    
    def isRoad(self,type):
        return type < self.ROADS
    
    def outOfBounds(self,x,y):
        return x < 0 or x >= self.width or y < 0 or y >= self.height
    
    def prettyPrint(self):
        for row in self.rows():
            print row
    
    def row(self,y):
        minSlice = y*self.width
        maxSlice = (y+1) * self.width
        return self.data[minSlice:maxSlice]
    
    def rows(self):
        for y in range(self.height):
            yield self.row(y)
            
    def tileFromWorld(self,worldX,worldY):
        width = self.width * common.BLOCK_UNITS
        height = self.height * common.BLOCK_UNITS
        upper = (height / 2.0)
        left = -(width/2.0)
        
        tX = (worldX - left) / common.BLOCK_UNITS
        tY = -((worldY - upper) / common.BLOCK_UNITS)
        if tX < 0: tX = -1
        if tY < 0: tY = -1
        return int(tX),int(tY)
            
    def tileToWorld(self,tileX,tileY):
        width = self.width * common.BLOCK_UNITS
        height = self.height * common.BLOCK_UNITS
        upper = (height / 2.0)
        left = -(width/2.0)
        
        x1 = left + tileX * common.BLOCK_UNITS
        y1 = upper + (-tileY * common.BLOCK_UNITS)
        return x1,y1
    
    def tileToWorldCentered(self,tileX,tileY):
        zerozero = self.tileToWorld(tileX,tileY)
        return (zerozero[0] + common.BLOCK_UNITS / 2,
            zerozero[1] - common.BLOCK_UNITS / 2)
    
        