
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.constants import *

import common
import sprites

# OpenGL font library based heavily on 
# glFreeType.py by Brian Leair,
# courtesy NeHe tutorials at:
#   http://nehe.gamedev.net/data/lessons/lesson.asp?lesson=43

class GLFont:
    def __init__(self,filename,size=14):
        self.font = pygame.font.Font(filename,size)
        self.listBase = glGenLists(128)
        self.textures = [None] * 128
        
        for i in xrange(128):
            self.processChar(i)
        
    def __del__(self):
        self.release()
        
    def glRender(self,x,y,string):
        "Assumes you've done typical gluOrtho2d thing"
        lines = string.split("\n")
        
        glListBase(self.listBase)
        
        h = self.font.get_linesize()
        
        for i in range(len(lines)):
            line = lines[i]
            modelview_matrix = glGetFloatv(GL_MODELVIEW_MATRIX)
            glPushMatrix()
            glLoadIdentity()
            glTranslatef(x, y-h*i, 0)
            glMultMatrixf(modelview_matrix)
            glCallLists(line)
            glPopMatrix()
            
    def processChar(self,char):
        try:
            glyph = self.font.render(chr(char),0,[255,255,255])
            width = common.nextPowerOf2(glyph.get_width() + 1)
            height = common.nextPowerOf2(glyph.get_height() + 1)
            
            texture = pygame.Surface((width,height),SRCALPHA,32)
            texture.blit(glyph, (0,height - glyph.get_height()))
            
            expanded_data = pygame.image.tostring(texture,"RGBA",1)
            
            if width and height:
                id = glGenTextures(1)
                self.textures[char] = id
                glBindTexture(GL_TEXTURE_2D, id)
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
                
                
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0,
                              # GL_RGBA for texture, GL_LUMINANCE_ALPHA for manual
                              GL_RGBA, GL_UNSIGNED_BYTE, expanded_data)
                
                glNewList(self.listBase + char, GL_COMPILE)
                if(char == ord(' ')):
                    glyph_advance = glyph.get_width()
                    glTranslatef(glyph_advance, 0, 0)
                    glEndList()
                else:
                    glBindTexture(GL_TEXTURE_2D, id)
                    glPushMatrix()
                    x = float(glyph.get_width()) / float(width)
                    y = float(glyph.get_height()) / float(height)
                    glBegin(GL_QUADS)
                    glTexCoord2f(0,0); glVertex2f(0,glyph.get_height())
                    glTexCoord2f(0,y); glVertex2f(0,0)
                    glTexCoord2f(x,y); glVertex2f(glyph.get_width(), 0)
                    glTexCoord2f(x,0); glVertex2f(glyph.get_width(), glyph.get_height())
                    glEnd()
                    glPopMatrix()
                
                    glTranslatef(glyph.get_width() + 0.75, 0, 0)
                    
                    glEndList()
        except pygame.error:
            pass
            
    def release(self):
        for id in self.textures:
            glDeleteTextures(id)