#!/usr/bin/env python

# We don't actually use these, but we do need to 
# check them on startup, so here they are.
try:
    from OpenGL.GL import *
    from OpenGL.GLU import *
except:
    print 'Getaway requires PyOpenGL... quitting.'
    raise SystemExit

import pygame

from pygame.constants import *

from engine import *

import title

class Dispatcher:
    
    def __init__(self, robber):
        self.robber = robber
        self.robber.dispatcher = self
        self.cops = []
        
    def addCop(self,cop):
        self.cops.append(cop)
        cop.dispatcher = self
        
    def collideDetect(self):
        checklist = [self.robber] + self.cops
        for startIndex in range(len(checklist)-1):
            subject = checklist[startIndex]
            for target in checklist[startIndex+1:]:
                subject.collideCheck(target)
                
    def robberBeingTailed(self):
        for cop in self.cops:
            if cop.mode == cop.MODE_CHASE:
                return True
        return False
                
    def update(self,delay):
        self.collideDetect()

class GetawayEngine(Engine):
    
    def __init__(self,screen,car):
        Engine.__init__(self,screen)
        self.car = car
        car.engine = self
        self.screen.add(car)
        self.updateables.append(car)
        
    def gameOver(self):
        self.updateables = []
        
    def keyEvent(self,event,pressed):
        if event.key == K_PAGEUP and pressed:
            self.screen.zoomIn()
        elif event.key == K_PAGEDOWN and pressed:
            self.screen.zoomOut()
        elif event.key == K_UP:
            self.car.accelerate(pressed)
        elif event.key == K_DOWN:
            self.car.decelerate(pressed)
        elif event.key == K_LEFT:
            self.car.turnLeft(pressed)
        elif event.key == K_RIGHT:
            self.car.turnRight(pressed)
        elif event.key == K_c and pressed:
            self.screen.toggleCameraMode()
        elif event.key == K_ESCAPE:
            self.running = False
        elif event.key == K_SPACE:
            if len(self.updateables) == 0:
                # Game's over, start title back up
                self.screen.clear()
                engine = title.TitleEngine(self.screen)
                self.running = False
                engine.run()

def main():
    pygame.font.init()
    screen = GLScreen(640,480)
    screen.realize()
    
    pygame.display.set_caption("The Getaway!")
    
    engine = title.TitleEngine(screen)   
    engine.run()
    
if __name__=='__main__':
    main()