from OpenGL.GL import *
from OpenGL.GLU import *

import os

class Importer:
    def __init__(self):
        pass

    def process(self,filename,scale):
        pass

class Material:
    
    def __init__(self,name):
        self.name = name
        self.ambient = ( 0.2, 0.2, 0.2 )
        self.diffuse = ( 0.8, 0.8, 0.8 )
        self.specular = ( 1.0, 1.0, 1.0 )
        self.transparent = 1.0
        self.shiny = 0.0
        self.useSpec = True
        self.texture = None
        
    def doGLCommands(self):
        ambient = self.ambient + [ self.transparent ]
        diffuse = self.diffuse + [ self.transparent ]
        specular = self.specular + [ self.specular ]
        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient)
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse)
        if self.useSpec:
            glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular)
        else:
            glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, (0,0,0))
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, self.shiny)
        
        if self.texture:
            #print "Using texture",self.texture
            glBindTexture(GL_TEXTURE_2D, self.texture)
        else:
            #print "Unbinding texture"
            glBindTexture(GL_TEXTURE_2D, 0)

class OBJImporter(Importer):
        
    def __init__(self, texLoader):
        Importer.__init__(self)
        self.vertices = []
        self.vertextures = []
        self.mtlMap = {}
        self.texLoader = texLoader
    
    def addFace(self,vlist):
        self.ensureList()
        
        glBegin(GL_POLYGON)
        for index in vlist:
            indexList = index.split('/')
            
            if len(indexList) > 1:
                # texture
                texIndex = int(indexList[1])
                glTexCoord3fv(self.vertextures[texIndex])
            
            actualIndex = int(indexList[0])
            glVertex( self.vertices[actualIndex] )
                
        glEnd()
    
    def addVertexTextures(self,vlist):
        u = float(vlist[0])
        v = float(vlist[1])
        if len(vlist) == 3:
            w = float(vlist[2])
        else:
            w = 0.0
        self.vertextures.append( (u,v,w) )
    
    def addVertices(self,vlist):
        x = float(vlist[0])
        y = float(vlist[1])
        z = float(vlist[2])
        self.vertices.append( (x,y,z) )
    
    def ensureList(self):
        if(self.listId == None):
            self.listId = glGenLists(1)
            glNewList(self.listId, GL_COMPILE)
            glScale(self.scale,self.scale,self.scale)
    
    def parseMTL(self,filename):
        dir = os.path.dirname(self.filename)
        filename = os.path.join(dir,filename)
        
        mtl = open(filename, 'r')
        curMaterial = None
        
        for line in mtl:
            #print line,
            parts = line.split()
            if parts:
                command = parts[0]
                if command == '#':
                    continue
                elif command == 'newmtl':
                    if curMaterial:
                        self.mtlMap[curMaterial.name] = curMaterial
                    name = " ".join(parts[1:])
                    curMaterial = Material(name)
                elif command == 'Ka':
                    curMaterial.ambient = self.parseRGB(parts[1:])
                elif command == 'Kd':
                    curMaterial.diffuse = self.parseRGB(parts[1:])
                elif command == 'Ks':
                    curMaterial.specular = self.parseRGB(parts[1:])
                elif (command == 'd' or command == 'Tr'):
                    curMaterial.transparent = float(parts[1])
                elif command == 'Ns':
                    curMaterial.shiny = float(parts[1])
                elif command == 'illum':
                    if parts[1] == '1':
                        curMaterial.useSpec = False
                elif command == 'map_Kd':
                    filename = " ".join(parts[1:])
                    texName = os.path.join(dir,filename)  
                    if self.listId:
                        print "TEX LOADING IN A LIST!"
                    curMaterial.texture,w,h = self.texLoader.load(texName)
                    
        if curMaterial:
            self.mtlMap[curMaterial.name] = curMaterial        
            
    def parseRGB(self,rgbStrings):
        r, g, b = rgbStrings
        r = float(r)
        g = float(g)
        b = float(b)
        return [r,g,b]
                
    def process(self,filename,scale):
        "Filename is /without/ the extention"
        self.filename = filename
        self.scale = scale
        objName = filename + '.obj'
        
        # OBJ format doesn't zero-index, adding filler
        self.vertices = [ (0,0,0) ]
        self.vertextures = [ (0,0) ]
        
        f = open(objName,'r')
        
        self.listId = None
        #glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, [0.2,1.0,0.2,1.0])
        #glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, [1.0,0.2,0.2,1.0])
        
        for line in f:
            #print line,
            parts = line.split()
            if parts:
                command = parts[0]
                if command == '#':
                    continue
                elif command == 'v':
                    self.addVertices(parts[1:])
                elif command == 'vt':
                    self.addVertexTextures(parts[1:])
                elif command == 'f':
                    self.addFace(parts[1:])
                elif command == 'mtllib':
                    self.parseMTL(" ".join(parts[1:]))
                elif command == 'usemtl':
                    self.useMaterial(" ".join(parts[1:]))
    
        glEndList();
        return self.listId

    def useMaterial(self,name):
        self.ensureList()
        material = self.mtlMap[name]
        material.doGLCommands()