import os

import sprites
from engine import *
from city import *
from title import *

from getaway import *
from sprites import Label
from font import *

class TitleEngine(Engine):
    
    def __init__(self,screen):
        Engine.__init__(self,screen)
        bigfont = GLFont(os.path.join("font","freesansbold.ttf"),72)
        smallfont = GLFont(os.path.join("font","freesansbold.ttf"),14)
        screen.addToHud(Label("The Getaway!",bigfont,40,40))
    
        self.size = 16
        self.cops = 5
        self.sizeLabel = Label("City Size: 16x16",smallfont,50,200)
        self.copsLabel = Label("Police: 5",smallfont,50,220)
        self.startLabel = Label("Start",smallfont,50,240)

        self.menuChoices = [self.sizeLabel, self.copsLabel,
                             self.startLabel]
        for choice in self.menuChoices:
            screen.addToHud(choice)
        
        self.choiceLabel = Label(">", smallfont,40,200)
        self.chosenIndex = 0
        screen.addToHud(self.choiceLabel)
        
        self.image = sprites.Image(
            os.path.join("images","streets.png"),
            screen.width - 276, screen.height - 276)
        screen.addToHud(self.image)
        
        glColor(1.0, 1.0, 1.0)
        
    def adjustSelected(self,increase):
        if not increase: increase = -1
        if self.chosenIndex == 0:
            dSize = common.sign(increase) * 3
            result = self.size + dSize
            if result >= 4:
                self.size = result
                self.sizeLabel.text="City Size: %dx%d" %(result,result)
        elif self.chosenIndex == 1:
            dCops = common.sign(increase)
            result = self.cops + dCops
            if result >= 0:
                self.cops = result
                self.copsLabel.text = "Police: %d" % result
    
    def adjustSelection(self,moveDown):
        if not moveDown: moveDown = -1
        self.chosenIndex = (self.chosenIndex + moveDown) % 3
        self.choiceLabel.y = 200 + (20 * self.chosenIndex)
        
    def keyEvent(self,event,pressed):
        if (event.key == K_RETURN or 
             event.key == K_KP_ENTER):
            self.startGame()
        if (pressed and (event.key == K_LEFT or
             event.key == K_RIGHT)):
            self.adjustSelected(event.key == K_RIGHT)
        if (pressed and (event.key == K_UP or
             event.key == K_DOWN)):
            self.adjustSelection(event.key == K_DOWN)
        
    def startGame(self):
        
        self.screen.clear()
        
        city = City([])
        city.createGridCity(self.size,self.size)
        
        you = sprites.PlayerCar(city,self.screen)
        
        hud = sprites.Hud(city,you)
        you.hud = hud
        dispatcher = Dispatcher(you)
        
        engine = GetawayEngine(self.screen,you)
        self.screen.tracking = you
        self.screen.addToHud(hud)
        x,y = city.getRandomRoad();
        you.location.move(x,y)
    
        # Cheese it!
        for x in range(self.cops):
            police = sprites.PoliceCar(city, you,self.screen)
            police.location.move(*city.getRandomRoad())
            engine.add(police)
            dispatcher.addCop(police)
    
        engine.add(dispatcher,False)
    
        ground = sprites.Ground(city)
        self.screen.add(ground)
    
        # Transition!
        self.running = False
        engine.run()
        